@@ -29,7 +29,6 @@ THE SOFTWARE.
 
 #include "OgreAutoParamDataSource.h"
 #include "OgreRenderable.h"
-#include "OgreRenderTarget.h"
 #include "OgreControllerManager.h"
 #include "OgreViewport.h"
 
@@ -52,12 +51,13 @@ namespace Ogre {
          mCameraPositionDirty(true),
          mCameraPositionObjectSpaceDirty(true),
          mAmbientLight(ColourValue::Black),
+         mShadowColour(ColourValue(0.25, 0.25, 0.25)),
          mPassNumber(0),
          mSceneDepthRangeDirty(true),
          mLodCameraPositionDirty(true),
          mLodCameraPositionObjectSpaceDirty(true),
          mCurrentRenderable(0),
-         mCurrentCamera(0), 
+         mCurrentCamera(0),
          mCameraRelativeRendering(false),
          mCurrentLightList(0),
          mCurrentRenderTarget(0),
@@ -70,7 +70,7 @@ namespace Ogre {
         mBlankLight.setDiffuseColour(ColourValue::Black);
         mBlankLight.setSpecularColour(ColourValue::Black);
         mBlankLight.setAttenuation(0,1,0,0);
-        mBlankLight._notifyAttached(&mDummyNode);
+        mDummyNode.attachObject(&mBlankLight);
         for(size_t i = 0; i < OGRE_MAX_SIMULTANEOUS_LIGHTS; ++i)
         {
             mTextureViewProjMatrixDirty[i] = true;
@@ -143,6 +143,10 @@ namespace Ogre {
         mLodCameraPositionObjectSpaceDirty = true;
         mLodCameraPositionDirty = true;
     }
+    void AutoParamDataSource::setCameraArray(const std::vector<const Camera*> cameras)
+    {
+        mCameraArray = cameras;
+    }
     //-----------------------------------------------------------------------------
     void AutoParamDataSource::setCurrentLightList(const LightList* ll)
     {
@@ -214,10 +218,16 @@ namespace Ogre {
         return getLight(index).getPowerScale();
     }
     //-----------------------------------------------------------------------------
-    const Vector4f& AutoParamDataSource::getLightAttenuation(size_t index) const
+    Vector4f AutoParamDataSource::getLightAttenuation(size_t index) const
     {
+        const Light& l = getLight(index);
+        if(l.getType() == Light::LT_RECTLIGHT)
+        {
+            auto rot = getViewMatrix().linear();
+            return Vector4f(Vector3f(rot * Vector3(l.getDerivedSourceHalfHeight())), 0.0);
+        }
         // range, const, linear, quad
-        return getLight(index).getAttenuation();
+        return l.getAttenuation();
     }
     //-----------------------------------------------------------------------------
     Vector4f AutoParamDataSource::getSpotlightParams(size_t index) const
@@ -231,6 +241,11 @@ namespace Ogre {
                            l.getSpotlightFalloff(),
                            1.0);
         }
+        else if(l.getType() == Light::LT_RECTLIGHT)
+        {
+            auto rot = getViewMatrix().linear();
+            return Vector4f(Vector3f(rot * Vector3(l.getDerivedSourceHalfWidth())), 2.0);
+        }
         else
         {
             // Use safe values which result in no change to point & dir light calcs
@@ -271,7 +286,8 @@ namespace Ogre {
             mWorldMatrixCount = mCurrentRenderable->getNumWorldTransforms();
             if (mCameraRelativeRendering && !mCurrentRenderable->getUseIdentityView())
             {
-                for (size_t i = 0; i < mWorldMatrixCount; ++i)
+                size_t worldMatrixCount = MeshManager::getBonesUseObjectSpace() ? 1 : mWorldMatrixCount;
+                for (size_t i = 0; i < worldMatrixCount; ++i)
                 {
                     mWorldMatrix[i].setTrans(mWorldMatrix[i].getTrans() - mCameraRelativePosition);
                 }
@@ -281,35 +297,40 @@ namespace Ogre {
         return mWorldMatrixArray[0];
     }
     //-----------------------------------------------------------------------------
-    size_t AutoParamDataSource::getWorldMatrixCount(void) const
+    size_t AutoParamDataSource::getBoneMatrixCount(void) const
     {
         // trigger derivation
         getWorldMatrix();
-        return mWorldMatrixCount;
+        return mWorldMatrixCount == 1 ? 1 : mWorldMatrixCount - int(MeshManager::getBonesUseObjectSpace());
     }
     //-----------------------------------------------------------------------------
-    const Affine3* AutoParamDataSource::getWorldMatrixArray(void) const
+    const Affine3* AutoParamDataSource::getBoneMatrixArray(void) const
     {
         // trigger derivation
         getWorldMatrix();
-        return mWorldMatrixArray;
+        return mWorldMatrixArray + int(MeshManager::getBonesUseObjectSpace());
     }
     //-----------------------------------------------------------------------------
+    Affine3 AutoParamDataSource::getViewMatrix(const Camera* cam) const
+    {
+        Affine3 view;
+        if (mCurrentRenderable && mCurrentRenderable->getUseIdentityView())
+            view = Affine3::IDENTITY;
+        else
+        {
+            view = cam->getViewMatrix(true);
+            if (mCameraRelativeRendering)
+            {
+                view.setTrans(Vector3::ZERO);
+            }
+        }
+        return view;
+    }
     const Affine3& AutoParamDataSource::getViewMatrix(void) const
     {
         if (mViewMatrixDirty)
         {
-            if (mCurrentRenderable && mCurrentRenderable->getUseIdentityView())
-                mViewMatrix = Affine3::IDENTITY;
-            else
-            {
-                mViewMatrix = mCurrentCamera->getViewMatrix(true);
-                if (mCameraRelativeRendering)
-                {
-                    mViewMatrix.setTrans(Vector3::ZERO);
-                }
-
-            }
+            mViewMatrix = getViewMatrix(mCurrentCamera);
             mViewMatrixDirty = false;
         }
         return mViewMatrix;
@@ -325,31 +346,40 @@ namespace Ogre {
         return mViewProjMatrix;
     }
     //-----------------------------------------------------------------------------
+    Matrix4 AutoParamDataSource::getProjectionMatrix(const Camera* cam) const
+    {
+        Matrix4 proj;
+
+        // NB use API-independent projection matrix since GPU programs
+        // bypass the API-specific handedness and use right-handed coords
+        if (mCurrentRenderable && mCurrentRenderable->getUseIdentityProjection())
+        {
+            // Use identity projection matrix, still need to take RS depth into account.
+            RenderSystem* rs = Root::getSingleton().getRenderSystem();
+            rs->_convertProjectionMatrix(Matrix4::IDENTITY, proj, true);
+        }
+        else
+        {
+            proj = cam->getProjectionMatrixWithRSDepth();
+        }
+
+        if (mCurrentRenderTarget && mCurrentRenderTarget->requiresTextureFlipping())
+        {
+            // Because we're not using setProjectionMatrix, this needs to be done here
+            // Invert transformed y
+            proj[1][0] = -proj[1][0];
+            proj[1][1] = -proj[1][1];
+            proj[1][2] = -proj[1][2];
+            proj[1][3] = -proj[1][3];
+        }
+
+        return proj;
+    }
     const Matrix4& AutoParamDataSource::getProjectionMatrix(void) const
     {
         if (mProjMatrixDirty)
         {
-            // NB use API-independent projection matrix since GPU programs
-            // bypass the API-specific handedness and use right-handed coords
-            if (mCurrentRenderable && mCurrentRenderable->getUseIdentityProjection())
-            {
-                // Use identity projection matrix, still need to take RS depth into account.
-                RenderSystem* rs = Root::getSingleton().getRenderSystem();
-                rs->_convertProjectionMatrix(Matrix4::IDENTITY, mProjectionMatrix, true);
-            }
-            else
-            {
-                mProjectionMatrix = mCurrentCamera->getProjectionMatrixWithRSDepth();
-            }
-            if (mCurrentRenderTarget && mCurrentRenderTarget->requiresTextureFlipping())
-            {
-                // Because we're not using setProjectionMatrix, this needs to be done here
-                // Invert transformed y
-                mProjectionMatrix[1][0] = -mProjectionMatrix[1][0];
-                mProjectionMatrix[1][1] = -mProjectionMatrix[1][1];
-                mProjectionMatrix[1][2] = -mProjectionMatrix[1][2];
-                mProjectionMatrix[1][3] = -mProjectionMatrix[1][3];
-            }
+            mProjectionMatrix = getProjectionMatrix(mCurrentCamera);
             mProjMatrixDirty = false;
         }
         return mProjectionMatrix;
@@ -374,6 +404,18 @@ namespace Ogre {
         }
         return mWorldViewProjMatrix;
     }
+    Matrix4 AutoParamDataSource::getWorldViewProjMatrix(size_t index) const
+    {
+        if (index >= mCameraArray.size())
+        {
+            return Matrix4::IDENTITY;
+        }
+
+        // dont bother caching this, as it invalidates per object
+        const auto& projectionMatrix = getProjectionMatrix(mCameraArray[index]);
+        const auto& viewMatrix = getViewMatrix(mCameraArray[index]);
+        return projectionMatrix * viewMatrix * getWorldMatrix();
+    }
     //-----------------------------------------------------------------------------
     const Affine3& AutoParamDataSource::getInverseWorldMatrix(void) const
     {
@@ -547,9 +589,10 @@ namespace Ogre {
                 static_cast<unsigned short>(index))->_getTexturePtr();
             if (tex)
             {
-                size[0] = static_cast<Real>(tex->getWidth());
-                size[1] = static_cast<Real>(tex->getHeight());
-                size[2] = static_cast<Real>(tex->getDepth());
+                size[0] = static_cast<float>(tex->getWidth());
+                size[1] = static_cast<float>(tex->getHeight());
+                size[2] = static_cast<float>(tex->getDepth());
+                size[3] = static_cast<float>(tex->getNumMipmaps());
             }
         }
 
@@ -600,7 +643,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     ColourValue AutoParamDataSource::getDerivedAmbientLightColour(void) const
     {
-        return getAmbientLightColour() * getSurfaceAmbientColour();
+        auto result = getAmbientLightColour() * getSurfaceAmbientColour();
+        result.a = getSurfaceDiffuseColour().a;
+        return result;
     }
     //-----------------------------------------------------------------------------
     ColourValue AutoParamDataSource::getDerivedSceneColour(void) const
@@ -657,9 +702,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     const Matrix4& AutoParamDataSource::getTextureViewProjMatrix(size_t index) const
     {
-        if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+        if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS && mCurrentTextureProjector[index])
         {
-            if (mTextureViewProjMatrixDirty[index] && mCurrentTextureProjector[index])
+            if (mTextureViewProjMatrixDirty[index])
             {
                 if (mCameraRelativeRendering)
                 {
@@ -678,7 +723,7 @@ namespace Ogre {
                     mTextureViewProjMatrix[index] = 
                         Matrix4::CLIPSPACE2DTOIMAGESPACE *
                         mCurrentTextureProjector[index]->getProjectionMatrixWithRSDepth() * 
-                        mCurrentTextureProjector[index]->getViewMatrix();
+                        mCurrentTextureProjector[index]->Frustum::getViewMatrix();
                 }
                 mTextureViewProjMatrixDirty[index] = false;
             }
@@ -690,9 +735,9 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     const Matrix4& AutoParamDataSource::getTextureWorldViewProjMatrix(size_t index) const
     {
-        if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+        if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS && mCurrentTextureProjector[index])
         {
-            if (mTextureWorldViewProjMatrixDirty[index] && mCurrentTextureProjector[index])
+            if (mTextureWorldViewProjMatrixDirty[index])
             {
                 mTextureWorldViewProjMatrix[index] = 
                     getTextureViewProjMatrix(index) * getWorldMatrix();
@@ -736,15 +781,7 @@ namespace Ogre {
                     // Use camera up
                     up = Vector3::UNIT_Z;
                 }
-                // cross twice to rederive, only direction is unaltered
-                Vector3 left = dir.crossProduct(up);
-                left.normalise();
-                up = dir.crossProduct(left);
-                up.normalise();
-                // Derive quaternion from axes
-                Quaternion q;
-                q.FromAxes(left, up, dir);
-                dummyNode.setOrientation(q);
+                dummyNode.setOrientation(Math::lookRotation(dir, up));
 
                 // The view matrix here already includes camera-relative changes if necessary
                 // since they are built into the frustum position
@@ -1028,17 +1065,17 @@ namespace Ogre {
         return mCurrentCamera->getDerivedUp();
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getFOV() const
+    float AutoParamDataSource::getFOV() const
     { 
         return mCurrentCamera->getFOVy().valueRadians(); 
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getNearClipDistance() const
+    float AutoParamDataSource::getNearClipDistance() const
     { 
         return mCurrentCamera->getNearClipDistance(); 
     }
     //-----------------------------------------------------------------------------
-    Real AutoParamDataSource::getFarClipDistance() const
+    float AutoParamDataSource::getFarClipDistance() const
     { 
         return mCurrentCamera->getFarClipDistance(); 
     }
@@ -1057,6 +1094,10 @@ namespace Ogre {
     {
         ++mPassNumber;
     }
+    int AutoParamDataSource::getMaterialLodIndex() const
+    {
+        return mCurrentRenderable->_getMaterialLodIndex();
+    }
     //-----------------------------------------------------------------------------
     const Vector4& AutoParamDataSource::getSceneDepthRange() const
     {
@@ -1087,7 +1128,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     const Vector4& AutoParamDataSource::getShadowSceneDepthRange(size_t index) const
     {
-        static Vector4 dummy(0, 100000, 100000, 1/100000);
+        static Vector4 dummy(0, 100000, 100000, 1.0f/100000);
 
         if (!mCurrentSceneManager->isShadowTechniqueTextureBased())
             return dummy;
@@ -1122,9 +1163,13 @@ namespace Ogre {
             return dummy;
     }
     //---------------------------------------------------------------------
+    void AutoParamDataSource::setShadowColour(const ColourValue& colour)
+    {
+        mShadowColour = colour;
+    }
     const ColourValue& AutoParamDataSource::getShadowColour() const
     {
-        return mCurrentSceneManager->getShadowColour();
+        return mShadowColour;
     }
     //-------------------------------------------------------------------------
     void AutoParamDataSource::updateLightCustomGpuParameter(const GpuProgramParameters::AutoConstantEntry& constantEntry, GpuProgramParameters *params) const
