@@ -29,32 +29,33 @@ THE SOFTWARE.
 #define __CgPlugin_H__
 
 #include "OgrePlugin.h"
+#include "OgreCgPrerequisites.h"
 
 namespace Ogre
 {
     class CgProgramFactory;
 
     /** Plugin instance for Cg Program Manager */
-    class CgPlugin : public Plugin
+    class _OgreCgPluginExport CgPlugin : public Plugin
     {
     public:
         CgPlugin();
 
 
         /// @copydoc Plugin::getName
-        const String& getName() const;
+        const String& getName() const override;
 
         /// @copydoc Plugin::install
-        void install();
+        void install() override;
 
         /// @copydoc Plugin::initialise
-        void initialise();
+        void initialise() override;
 
         /// @copydoc Plugin::shutdown
-        void shutdown();
+        void shutdown() override;
 
         /// @copydoc Plugin::uninstall
-        void uninstall();
+        void uninstall() override;
     protected:
         CgProgramFactory* mCgProgramFactory;
 
