@@ -47,7 +47,7 @@ namespace Ogre
         change only a few of the shaders in the pipeline while leaving
         the rest the same.
 
-        @remarks
+
         GLSL has no target assembler or entry point specification like
         DirectX 9 HLSL.  Vertex and Fragment shaders only have one
         entry point called "main".  When a shader is compiled,
@@ -83,26 +83,19 @@ namespace Ogre
             just before rendering occurs.
         */
         void updateUniforms(GpuProgramParametersSharedPtr params,
-                            uint16 mask, GpuProgramType fromProgType);
-        /** Updates program object atomic counter buffers using data
-            from GpuProgramParameters.  Normally called by
-            GLSLShader::bindProgramAtomicCounterParameters() just
-            before rendering occurs.
-        */
-        void updateAtomicCounters(GpuProgramParametersSharedPtr params,
-                                  uint16 mask, GpuProgramType fromProgType);
+                            uint16 mask, GpuProgramType fromProgType) override;
 
         /** Makes a program pipeline object active by making sure it
             is linked and then putting it in use.
         */
-        void activate(void);
+        void activate(void) override;
 
     protected:
         /// GL handle for pipeline object.
         GLuint mGLProgramPipelineHandle;
 
         /// Compiles and links the separate programs.
-        void compileAndLink(void);
+        void compileAndLink(void) override;
 
         void getMicrocodeFromCache(void);
         void getIndividualProgramMicrocodeFromCache(GLSLShader* program);
