@@ -50,39 +50,37 @@ namespace Ogre {
 
             ::EGLSurface createSurfaceFromWindow(::EGLDisplay display, NativeWindowType win);
 
-            virtual void switchFullScreen(bool fullscreen) = 0;
+            virtual void switchFullScreen(bool fullscreen) {}
             EGLContext * createEGLContext(::EGLContext external = NULL) const {
                 return new EGLContext(mEglDisplay, mGLSupport, mEglConfig, mEglSurface, external);
             }
 
-            virtual void getLeftAndTopFromNativeWindow(int & left, int & top, uint width, uint height) = 0;
-            virtual void initNativeCreatedWindow(const NameValuePairList *miscParams) = 0;
-            virtual void createNativeWindow( int &left, int &top, uint &width, uint &height, String &title ) = 0;
-            virtual void windowMovedOrResized() = 0;
+            void windowMovedOrResized() override {}
+
+            void finaliseWindow();
     public:
             EGLWindow(EGLSupport* glsupport);
             virtual ~EGLWindow();
 
-//      Moved create to native source because it has native calls in it.
-//            void create(const String& name, unsigned int width, unsigned int height,
-//                        bool fullScreen, const NameValuePairList *miscParams);
+            // default, PBuffer based, implementation
+            void create(const String& name, unsigned int width, unsigned int height, bool fullScreen,
+                        const NameValuePairList* miscParams) override;
 
-            virtual void setFullscreen (bool fullscreen, uint width, uint height);
-            void destroy(void);
-            void swapBuffers();
-            void setVSyncEnabled(bool vsync);
+            void setFullscreen (bool fullscreen, uint width, uint height) override;
+            void destroy(void) override;
+            void swapBuffers() override;
+            void setVSyncEnabled(bool vsync) override;
 
             /**
-               @remarks
+
                * Get custom attribute; the following attributes are valid:
                * WINDOW         The X NativeWindowType target for rendering.
                * GLCONTEXT      The Ogre GLContext used for rendering.
                * DISPLAY        EGLDisplay connection behind that context.
-               * DISPLAYNAME    The name for the connected display.
                */
-            virtual void getCustomAttribute(const String& name, void* pData);
+            void getCustomAttribute(const String& name, void* pData) override;
 
-            PixelFormat suggestPixelFormat() const;
+            PixelFormat suggestPixelFormat() const override;
     };
 }
 
